@@ -1,5 +1,5 @@
 /* the Music Player Daemon (MPD)
- * (c)2003-2006 by Warren Dukes (warren.dukes@gmail.com)
+ * Copyright (C) 2003-2007 by Warren Dukes (warren.dukes@gmail.com)
  * This project's homepage is: http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -32,6 +32,7 @@
 #define CONF_BIND_TO_ADDRESS            "bind_to_address"
 #define CONF_PORT                       "port"
 #define CONF_LOG_LEVEL                  "log_level"
+#define CONF_ZEROCONF_NAME              "zeroconf_name"
 #define CONF_PASSWORD                   "password"
 #define CONF_DEFAULT_PERMS              "default_permissions"
 #define CONF_AUDIO_OUTPUT               "audio_output"
@@ -42,6 +43,7 @@
 #define CONF_REPLAYGAIN                 "replaygain"
 #define CONF_REPLAYGAIN_PREAMP          "replaygain_preamp"
 #define CONF_VOLUME_NORMALIZATION       "volume_normalization"
+#define CONF_SAMPLERATE_CONVERTER       "samplerate_converter"
 #define CONF_AUDIO_BUFFER_SIZE          "audio_buffer_size"
 #define CONF_BUFFER_BEFORE_PLAY         "buffer_before_play"
 #define CONF_HTTP_BUFFER_SIZE           "http_buffer_size"
@@ -59,6 +61,7 @@
 #define CONF_ID3V1_ENCODING             "id3v1_encoding"
 #define CONF_METADATA_TO_USE            "metadata_to_use"
 #define CONF_SAVE_ABSOLUTE_PATHS        "save_absolute_paths_in_playlists"
+#define CONF_GAPLESS_MP3_PLAYBACK	"gapless_mp3_playback"
 
 typedef struct _BlockParam {
 	char *name;
