@@ -1,5 +1,5 @@
 /* the Music Player Daemon (MPD)
- * (c)2003-2006 by Warren Dukes (warren.dukes@gmail.com)
+ * Copyright (C) 2003-2007 by Warren Dukes (warren.dukes@gmail.com)
  * This project's homepage is: http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -37,7 +37,7 @@
 #include <unistd.h>
 #include <string.h>
 
-static int decode_pid = 0;
+static int decode_pid;
 
 void decodeSigHandler(int sig, siginfo_t * si, void *v)
 {
@@ -76,7 +76,6 @@ static void stopDecode(DecoderControl * dc)
 static void quitDecode(PlayerControl * pc, DecoderControl * dc)
 {
 	stopDecode(dc);
-	pc->metadataState = PLAYER_METADATA_STATE_READ;
 	pc->state = PLAYER_STATE_STOP;
 	dc->seek = 0;
 	pc->play = 0;
@@ -221,20 +220,20 @@ static int decodeSeek(PlayerControl * pc, DecoderControl * dc,
 	} \
 	if(pc->pause) { \
 		pause = !pause; \
-		if(pause) pc->state = PLAYER_STATE_PAUSE; \
+		if (pause) pc->state = PLAYER_STATE_PAUSE; \
 		else { \
-			if(openAudioDevice(NULL)<0) { \
+			if (openAudioDevice(NULL) >= 0) pc->state = PLAYER_STATE_PLAY; \
+			else { \
 				pathcpy_trunc(pc->erroredUrl, pc->utf8url); \
 				pc->error = PLAYER_ERROR_AUDIO; \
 				ERROR("problems opening audio device while playing \"%s\"\n", pc->utf8url); \
-				quitDecode(pc,dc); \
-				return; \
+				pause = -1; \
 			} \
-			pc->state = PLAYER_STATE_PLAY; \
 		} \
 		pc->pause = 0; \
-		kill(getppid(),SIGUSR1); \
-		if(pause) { \
+		kill(getppid(), SIGUSR1); \
+		if (pause == -1) pause = 1; \
+		else if (pause) { \
 			dropBufferedAudio(); \
 			closeAudioDevice(); \
 		} \
@@ -285,7 +284,6 @@ static void decodeStart(PlayerControl * pc, OutputBuffer * cb,
 		return;
 	}
 
-	dc->seekable = inStream.seekable;
 	dc->state = DECODE_STATE_START;
 	dc->start = 0;
 
@@ -295,6 +293,9 @@ static void decodeStart(PlayerControl * pc, OutputBuffer * cb,
 		my_usleep(1000);
 	}
 
+	/* for http streams, seekable is determined in bufferInputStream */
+	dc->seekable = inStream.seekable;
+        
 	if (dc->stop) {
 		dc->state = DECODE_STATE_STOP;
 		dc->stop = 0;
@@ -480,10 +481,10 @@ static void advanceOutputBufferTo(OutputBuffer * cb, PlayerControl * pc,
 	while (cb->begin != to) {
 		handleMetadata(cb, pc, previous, currentChunkSent,
 			       currentChunk);
-		cb->begin++;
-		if (cb->begin >= buffered_chunks) {
+		if (cb->begin + 1 >= buffered_chunks) {
 			cb->begin = 0;
 		}
+		else cb->begin++;
 	}
 }
 
@@ -516,7 +517,8 @@ static void decodeParent(PlayerControl * pc, DecoderControl * dc, OutputBuffer *
 	pc->play = 0;
 	kill(getppid(), SIGUSR1);
 
-	while (decode_pid > 0 && cb->end - cb->begin < bbp &&
+	while (decode_pid > 0 && 
+	       cb->end - cb->begin < bbp &&
 	       cb->end != buffered_chunks - 1 &&
 	       dc->state != DECODE_STATE_STOP) {
 		processDecodeInput();
@@ -617,10 +619,10 @@ static void decodeParent(PlayerControl * pc, DecoderControl * dc, OutputBuffer *
 				cb->begin = 0;
 			} else
 				cb->begin++;
-		} else if (next == cb->begin) {
+		} else if (cb->begin != end && cb->begin == next) {
 			if (doCrossFade == 1 && nextChunk >= 0) {
 				nextChunk = cb->begin + crossFadeChunks;
-				test = cb->end;
+				test = end;
 				if (end < cb->begin)
 					test += buffered_chunks;
 				if (nextChunk < test) {
@@ -663,6 +665,7 @@ static void decodeParent(PlayerControl * pc, DecoderControl * dc, OutputBuffer *
 			quit = 1;
 			break;
 		} else {
+			/*DEBUG("waiting for decoded audio, play silence\n");*/
 			if (playAudio(silence, CHUNK_SIZE) < 0)
 				quit = 1;
 		}
