@@ -1,5 +1,5 @@
 /* the Music Player Daemon (MPD)
- * (c)2003-2006 by Warren Dukes (warren.dukes@gmail.com)
+ * Copyright (C) 2003-2007 by Warren Dukes (warren.dukes@gmail.com)
  * This project's homepage is: http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -59,15 +59,15 @@
 #define DIRECTORY_RETURN_UPDATE         1
 #define DIRECTORY_RETURN_ERROR         -1
 
-Directory *mp3rootDirectory = NULL;
+static Directory *mp3rootDirectory;
 
-time_t directory_dbModTime = 0;
+static time_t directory_dbModTime;
 
-volatile int directory_updatePid = 0;
+static volatile int directory_updatePid;
 
-volatile int directory_reReadDB = 0;
+static volatile int directory_reReadDB;
 
-volatile mpd_uint16 directory_updateJobId = 0;
+static volatile mpd_uint16 directory_updateJobId;
 
 static DirectoryList *newDirectoryList();
 
@@ -543,7 +543,7 @@ static int updatePath(char *utf8path)
 			free(path);
 			return 0;
 		}
-		/* if updateDirectory fials, means we should delete it */
+		/* if updateDirectory fails, means we should delete it */
 		else {
 			LOG("removing directory: %s\n", path);
 			deleteFromList(parentDirectory->subDirectories,
@@ -573,7 +573,7 @@ static int updatePath(char *utf8path)
 				return 1;
 			}
 		}
-		/* if updateDirectory fials, means we should delete it */
+		/* if updateDirectory fails, means we should delete it */
 		else {
 			removeSongFromDirectory(parentDirectory, shortname);
 			ret = 1;
@@ -835,13 +835,21 @@ static Directory *findSubDirectory(Directory * directory, char *name)
 	return NULL;
 }
 
+int isRootDirectory(char *name)
+{
+	if (name == NULL || name[0] == '\0' || strcmp(name, "/") == 0) {
+		return 1;
+	}
+	return 0;
+}
+
 static Directory *getSubDirectory(Directory * directory, char *name,
 				  char **shortname)
 {
 	Directory *subDirectory;
 	int len;
 
-	if (name == NULL || name[0] == '\0' || strcmp(name, "/") == 0) {
+	if (isRootDirectory(name)) {
 		return directory;
 	}
 
@@ -943,23 +951,18 @@ static void readDirectoryInfo(FILE * fp, Directory * directory)
 	       && 0 != strncmp(DIRECTORY_END, buffer, strlen(DIRECTORY_END))) {
 		if (0 == strncmp(DIRECTORY_DIR, buffer, strlen(DIRECTORY_DIR))) {
 			key = xstrdup(&(buffer[strlen(DIRECTORY_DIR)]));
-			if (!myFgets(buffer, bufferSize, fp)) {
-				ERROR("Error reading db, fgets\n");
-				exit(EXIT_FAILURE);
-			}
+			if (!myFgets(buffer, bufferSize, fp))
+				FATAL("Error reading db, fgets\n");
 			/* for compatibility with db's prior to 0.11 */
 			if (0 == strncmp(DIRECTORY_MTIME, buffer,
 					 strlen(DIRECTORY_MTIME))) {
-				if (!myFgets(buffer, bufferSize, fp)) {
-					ERROR("Error reading db, fgets\n");
-					exit(EXIT_FAILURE);
-				}
+				if (!myFgets(buffer, bufferSize, fp))
+					FATAL("Error reading db, fgets\n");
 			}
 			if (strncmp
 			    (DIRECTORY_BEGIN, buffer,
 			     strlen(DIRECTORY_BEGIN))) {
-				ERROR("Error reading db at line: %s\n", buffer);
-				exit(EXIT_FAILURE);
+				FATAL("Error reading db at line: %s\n", buffer);
 			}
 			name = xstrdup(&(buffer[strlen(DIRECTORY_BEGIN)]));
 
@@ -993,8 +996,7 @@ static void readDirectoryInfo(FILE * fp, Directory * directory)
 		} else if (0 == strncmp(SONG_BEGIN, buffer, strlen(SONG_BEGIN))) {
 			readSongInfoIntoList(fp, directory->songs, directory);
 		} else {
-			ERROR("Unknown line in db: %s\n", buffer);
-			exit(EXIT_FAILURE);
+			FATAL("Unknown line in db: %s\n", buffer);
 		}
 	}
 
@@ -1091,6 +1093,7 @@ int writeDirectoryDB(void)
 {
 	FILE *fp;
 	char *dbFile = getDbFile();
+	struct stat st;
 
 	DEBUG("removing empty directories from DB\n");
 	deleteEmptyDirectoriesInDirectory(mp3rootDirectory);
@@ -1101,7 +1104,7 @@ int writeDirectoryDB(void)
 
 	DEBUG("writing DB\n");
 
-	while (!(fp = fopen(dbFile, "w")) && errno == EINTR) ;
+	while (!(fp = fopen(dbFile, "w")) && errno == EINTR);
 	if (!fp) {
 		ERROR("unable to write to db file \"%s\": %s\n",
 		      dbFile, strerror(errno));
@@ -1116,7 +1119,10 @@ int writeDirectoryDB(void)
 
 	writeDirectoryInfo(fp, mp3rootDirectory);
 
-	while (fclose(fp) && errno == EINTR) ;
+	while (fclose(fp) && errno == EINTR);
+
+	if (stat(dbFile, &st) == 0)
+		directory_dbModTime = st.st_mtime;
 
 	return 0;
 }
@@ -1131,7 +1137,7 @@ int readDirectoryDB(void)
 		mp3rootDirectory = newDirectory(NULL, NULL);
 	while (!(fp = fopen(dbFile, "r")) && errno == EINTR) ;
 	if (fp == NULL) {
-		ERROR("unable open db file \"%s\": %s\n",
+		ERROR("unable to open db file \"%s\": %s\n",
 		      dbFile, strerror(errno));
 		return -1;
 	}
@@ -1143,21 +1149,16 @@ int readDirectoryDB(void)
 		int foundFsCharset = 0;
 		int foundVersion = 0;
 
-		if (!myFgets(buffer, bufferSize, fp)) {
-			ERROR("Error reading db, fgets\n");
-			exit(EXIT_FAILURE);
-		}
+		if (!myFgets(buffer, bufferSize, fp))
+			FATAL("Error reading db, fgets\n");
 		if (0 == strcmp(DIRECTORY_INFO_BEGIN, buffer)) {
 			while (myFgets(buffer, bufferSize, fp) &&
 			       0 != strcmp(DIRECTORY_INFO_END, buffer)) {
 				if (0 == strncmp(DIRECTORY_MPD_VERSION, buffer,
 						 strlen(DIRECTORY_MPD_VERSION)))
 				{
-					if (foundVersion) {
-						ERROR("already found "
-						      "version in db\n");
-						exit(EXIT_FAILURE);
-					}
+					if (foundVersion)
+						FATAL("already found version in db\n");
 					foundVersion = 1;
 				} else if (0 ==
 					   strncmp(DIRECTORY_FS_CHARSET, buffer,
@@ -1166,20 +1167,13 @@ int readDirectoryDB(void)
 					char *fsCharset;
 					char *tempCharset;
 
-					if (foundFsCharset) {
-						WARNING("already found "
-							"fs charset in db\n");
-						exit(EXIT_FAILURE);
-					}
+					if (foundFsCharset)
+						FATAL("already found fs charset in db\n");
 
 					foundFsCharset = 1;
 
-					fsCharset =
-					    &(buffer
-					      [strlen(DIRECTORY_FS_CHARSET)]);
-					if ((tempCharset =
-					     getConfigParamValue
-					     (CONF_FS_CHARSET))
+					fsCharset = &(buffer[strlen(DIRECTORY_FS_CHARSET)]);
+					if ((tempCharset = getConfigParamValue(CONF_FS_CHARSET))
 					    && strcmp(fsCharset, tempCharset)) {
 						WARNING("Using \"%s\" for the "
 							"filesystem charset "
@@ -1190,10 +1184,8 @@ int readDirectoryDB(void)
 						setFsCharset(fsCharset);
 					}
 				} else {
-					ERROR
-					    ("directory: unknown line in db info: %s\n",
+					FATAL("directory: unknown line in db info: %s\n",
 					     buffer);
-					exit(EXIT_FAILURE);
 				}
 			}
 		} else {
@@ -1225,14 +1217,12 @@ void updateMp3Directory(void)
 		/* nothing updated */
 		return;
 	case 1:
-		if (writeDirectoryDB() < 0) {
+		if (writeDirectoryDB() < 0)
 			exit(EXIT_FAILURE);
-		}
 		break;
 	default:
 		/* something was updated and db should be written */
-		ERROR("problems updating music db\n");
-		exit(EXIT_FAILURE);
+		FATAL("problems updating music db\n");
 	}
 
 	return;
@@ -1311,16 +1301,11 @@ static void freeAllDirectoryStats(Directory * directory)
 
 void initMp3Directory(void)
 {
-	struct stat st;
-
 	mp3rootDirectory = newDirectory(NULL, NULL);
 	exploreDirectory(mp3rootDirectory);
 	freeAllDirectoryStats(mp3rootDirectory);
 	stats.numberOfSongs = countSongsIn(STDERR_FILENO, NULL);
 	stats.dbPlayTime = sumSongTimesIn(STDERR_FILENO, NULL);
-
-	if (stat(getDbFile(), &st) == 0)
-		directory_dbModTime = st.st_mtime;
 }
 
 static Song *getSongDetails(char *file, char **shortnameRet,
