@@ -1,5 +1,5 @@
 /* the Music Player Daemon (MPD)
- * (c)2003-2006 by Warren Dukes (warren.dukes@gmail.com)
+ * Copyright (C) 2003-2007 by Warren Dukes (warren.dukes@gmail.com)
  * This project's homepage is: http://www.musicpd.org
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -64,8 +64,13 @@ static MDRIVER drv_mpd = {
 	"MPD Output Driver v0.1",
 	0,
 	255,
-	"mpd",
-	NULL,
+#if (LIBMIKMOD_VERSION > 0x030106)
+	"mpd", /* Alias */
+#if (LIBMIKMOD_VERSION > 0x030200)
+	NULL,  /* CmdLineHelp */
+#endif
+	NULL,  /* CommandLine */
+#endif
 	mod_mpd_IsThere,
 	VC_SampleLoad,
 	VC_SampleUnload,
@@ -92,8 +97,8 @@ static MDRIVER drv_mpd = {
 	VC_VoiceRealVolume
 };
 
-static int mod_mikModInitiated = 0;
-static int mod_mikModInitError = 0;
+static int mod_mikModInitiated;
+static int mod_mikModInitError;
 
 static int mod_initMikMod(void)
 {
@@ -143,6 +148,9 @@ static mod_Data *mod_open(char *path)
 	if (!(moduleHandle = Player_Load(path, 128, 0)))
 		return NULL;
 
+	/* Prevent module from looping forever */
+	moduleHandle->loop = 0;
+
 	data = xmalloc(sizeof(mod_Data));
 
 	data->audio_buffer = xmalloc(MIKMOD_FRAME_SIZE);
@@ -177,6 +185,7 @@ static int mod_decode(OutputBuffer * cb, DecoderControl * dc, char *path)
 		return -1;
 	}
 
+	dc->totalTime = 0;
 	dc->audioFormat.bits = 16;
 	dc->audioFormat.sampleRate = 44100;
 	dc->audioFormat.channels = 2;
@@ -285,17 +294,6 @@ InputPlugin modPlugin = {
 
 #else
 
-InputPlugin modPlugin = {
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	NULL,
-	0,
-	NULL,
-	NULL
-};
+InputPlugin modPlugin;
 
-#endif /* HAVE_AUDIOFILE */
+#endif /* HAVE_MIKMOD */
