@@ -1,5 +1,5 @@
 /* the Music Player Daemon (MPD)
- * (c)2003-2006 by Warren Dukes (warren.dukes@gmail.com)
+ * Copyright (C) 2003-2007 by Warren Dukes (warren.dukes@gmail.com)
  * This project's homepage is: http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -25,6 +25,8 @@
 #include <sys/types.h>
 #include <unistd.h>
 #include <errno.h>
+#define _XOPEN_SOURCE 600
+#include <fcntl.h>
 
 void inputStream_initFile(void)
 {
@@ -46,6 +48,10 @@ int inputStream_fileOpen(InputStream * inStream, char *filename)
 	inStream->size = ftell(fp);
 	fseek(fp, 0, SEEK_SET);
 
+#ifdef POSIX_FADV_SEQUENTIAL
+	posix_fadvise(fileno(fp), (off_t)0, inStream->size, POSIX_FADV_SEQUENTIAL);
+#endif
+
 	inStream->data = fp;
 	inStream->seekFunc = inputStream_fileSeek;
 	inStream->closeFunc = inputStream_fileClose;
