@@ -1,5 +1,5 @@
 /* the Music Player Daemon (MPD)
- * (c)2003-2006 by Warren Dukes (warren.dukes@gmail.com)
+ * Copyright (C) 2003-2007 by Warren Dukes (warren.dukes@gmail.com)
  * This project's homepage is: http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -26,6 +26,7 @@
 #include "permission.h"
 #include "sllist.h"
 #include "utils.h"
+#include "ioops.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -55,20 +56,23 @@
 #define INTERFACE_MAX_OUTPUT_BUFFER_SIZE_DEFAULT	(8192*1024)
 
 /* set this to zero to indicate we have no possible interfaces */
-static int interface_max_connections = 0;	/*INTERFACE_MAX_CONNECTIONS_DEFAULT; */
+static int interface_max_connections;	/*INTERFACE_MAX_CONNECTIONS_DEFAULT; */
 static int interface_timeout = INTERFACE_TIMEOUT_DEFAULT;
 static size_t interface_max_command_list_size =
     INTERFACE_MAX_COMMAND_LIST_DEFAULT;
 static size_t interface_max_output_buffer_size =
     INTERFACE_MAX_OUTPUT_BUFFER_SIZE_DEFAULT;
 
+/* List of registered external IO handlers */
+static struct ioOps *ioList;
+
 /* maybe make conf option for this, or... 32 might be good enough */
 static long int interface_list_cache_size = 32;
 
 /* shared globally between all interfaces: */
-static struct strnode *list_cache = NULL;
-static struct strnode *list_cache_head = NULL;
-static struct strnode *list_cache_tail = NULL;
+static struct strnode *list_cache;
+static struct strnode *list_cache_head;
+static struct strnode *list_cache_tail;
 
 typedef struct _Interface {
 	char buffer[INTERFACE_MAX_BUFFER_LENGTH];
@@ -94,7 +98,7 @@ typedef struct _Interface {
 	int send_buf_alloc;	/* bytes actually allocated */
 } Interface;
 
-static Interface *interfaces = NULL;
+static Interface *interfaces;
 
 static void flushInterfaceBuffer(Interface * interface);
 
@@ -489,6 +493,7 @@ int doIOForInterfaces(void)
 {
 	fd_set rfds;
 	fd_set wfds;
+	fd_set efds;
 	struct timeval tv;
 	int i;
 	int selret;
@@ -500,12 +505,43 @@ int doIOForInterfaces(void)
 	while (1) {
 		fdmax = 0;
 
+		FD_ZERO( &rfds );
+		FD_ZERO( &wfds );
+		FD_ZERO( &efds );
 		addInterfacesReadyToReadAndListenSocketToFdSet(&rfds, &fdmax);
 		addInterfacesForBufferFlushToFdSet(&wfds, &fdmax);
 
-		selret = select(fdmax + 1, &rfds, &wfds, NULL, &tv);
+		/* Add fds for all registered IO handlers */
+		if( ioList ) {
+			struct ioOps *o = ioList;
+			while( o ) {
+				struct ioOps *current = o;
+				int fdnum;
+				assert( current->fdset );
+				fdnum = current->fdset( &rfds, &wfds, &efds );
+				if( fdmax < fdnum )
+					fdmax = fdnum;
+				o = o->next;
+			}
+		}
 
-		if (selret == 0 || (selret < 0 && errno == EINTR))
+		selret = select(fdmax + 1, &rfds, &wfds, &efds, &tv);
+
+		if (selret < 0 && errno == EINTR)
+			break;
+
+		/* Consume fds for all registered IO handlers */
+		if( ioList ) {
+			struct ioOps *o = ioList;
+			while( o ) {
+				struct ioOps *current = o;
+				assert( current->consume );
+				selret = current->consume( selret, &rfds, &wfds, &efds );
+				o = o->next;
+			}
+		}
+
+		if (selret == 0)
 			break;
 
 		if (selret < 0) {
@@ -549,10 +585,9 @@ void initInterfaces(void)
 	if (param) {
 		interface_timeout = strtol(param->value, &test, 10);
 		if (*test != '\0' || interface_timeout <= 0) {
-			ERROR("connection timeout \"%s\" is not a positive "
+			FATAL("connection timeout \"%s\" is not a positive "
 			      "integer, line %i\n", CONF_CONN_TIMEOUT,
 			      param->line);
-			exit(EXIT_FAILURE);
 		}
 	}
 
@@ -561,9 +596,8 @@ void initInterfaces(void)
 	if (param) {
 		interface_max_connections = strtol(param->value, &test, 10);
 		if (*test != '\0' || interface_max_connections <= 0) {
-			ERROR("max connections \"%s\" is not a positive integer"
+			FATAL("max connections \"%s\" is not a positive integer"
 			      ", line %i\n", param->value, param->line);
-			exit(EXIT_FAILURE);
 		}
 	} else
 		interface_max_connections = INTERFACE_MAX_CONNECTIONS_DEFAULT;
@@ -574,9 +608,8 @@ void initInterfaces(void)
 		interface_max_command_list_size = strtol(param->value,
 							 &test, 10);
 		if (*test != '\0' || interface_max_command_list_size <= 0) {
-			ERROR("max command list size \"%s\" is not a positive "
+			FATAL("max command list size \"%s\" is not a positive "
 			      "integer, line %i\n", param->value, param->line);
-			exit(EXIT_FAILURE);
 		}
 		interface_max_command_list_size *= 1024;
 	}
@@ -587,9 +620,8 @@ void initInterfaces(void)
 		interface_max_output_buffer_size = strtol(param->value,
 		                                          &test, 10);
 		if (*test != '\0' || interface_max_output_buffer_size <= 0) {
-			ERROR("max output buffer size \"%s\" is not a positive "
+			FATAL("max output buffer size \"%s\" is not a positive "
 			      "integer, line %i\n", param->value, param->line);
-			exit(EXIT_FAILURE);
 		}
 		interface_max_output_buffer_size *= 1024;
 	}
@@ -696,7 +728,7 @@ static void flushInterfaceBuffer(Interface * interface)
 
 int interfacePrintWithFD(int fd, char *buffer, int buflen)
 {
-	static int i = 0;
+	static int i;
 	int copylen;
 	Interface *interface;
 
@@ -795,3 +827,25 @@ static void printInterfaceOutBuffer(Interface * interface)
 
 	interface->send_buf_used = 0;
 }
+
+/* From ioops.h: */
+void registerIO( struct ioOps *ops )
+{
+	assert( ops != NULL );
+
+	ops->next = ioList;
+	ioList = ops;
+	ops->prev = NULL;
+	if( ops->next )
+		ops->next->prev = ops;
+}
+
+void deregisterIO( struct ioOps *ops )
+{
+	assert( ops != NULL );
+
+	if( ioList == ops )
+		ioList = ops->next;
+	else if( ops->prev != NULL )
+		ops->prev->next = ops->next;
+}
