@@ -1,5 +1,5 @@
 /* the Music Player Daemon (MPD)
- * (c)2003-2006 by Warren Dukes (warren.dukes@gmail.com)
+ * Copyright (C) 2003-2007 by Warren Dukes (warren.dukes@gmail.com)
  * This project's homepage is: http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -41,17 +41,19 @@
 #define DEFAULT_PORT	6600
 
 #define BINDERROR() do { \
-	ERROR("unable to bind port %u: %s\n", port, strerror(errno)); \
-	ERROR("maybe MPD is still running?\n"); \
+	FATAL("unable to bind port %u: %s\n" \
+	      "maybe MPD is still running?\n", \
+	      port, strerror(errno)); \
 } while (0);
 
-int *listenSockets = NULL;
-int numberOfListenSockets = 0;
+static int *listenSockets;
+static int numberOfListenSockets;
+static int boundPort;
 
 static int establishListen(unsigned int port,
                            struct sockaddr *addrp, socklen_t addrlen)
 {
-	int pf;
+	int pf = 0;
 	int sock;
 	int allowReuse = ALLOW_REUSE;
 
@@ -68,25 +70,20 @@ static int establishListen(unsigned int port,
 		pf = PF_UNIX;
 		break;
 	default:
-		ERROR("unknown address family: %i\n", addrp->sa_family);
-		exit(EXIT_FAILURE);
+		FATAL("unknown address family: %i\n", addrp->sa_family);
 	}
 
-	if ((sock = socket(pf, SOCK_STREAM, 0)) < 0) {
-		ERROR("socket < 0\n");
-		exit(EXIT_FAILURE);
-	}
+	if ((sock = socket(pf, SOCK_STREAM, 0)) < 0)
+		FATAL("socket < 0\n");
 
 	if (fcntl(sock, F_SETFL, fcntl(sock, F_GETFL) | O_NONBLOCK) < 0) {
-		ERROR("problems setting nonblocking on listen socket: %s\n",
+		FATAL("problems setting nonblocking on listen socket: %s\n",
 		      strerror(errno));
-		exit(EXIT_FAILURE);
 	}
 
 	if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (char *)&allowReuse,
 		       sizeof(allowReuse)) < 0) {
-		ERROR("problems setsockopt'ing: %s\n", strerror(errno));
-		exit(EXIT_FAILURE);
+		FATAL("problems setsockopt'ing: %s\n", strerror(errno));
 	}
 
 	if (bind(sock, addrp, addrlen) < 0) {
@@ -94,10 +91,8 @@ static int establishListen(unsigned int port,
 		return -1;
 	}
 
-	if (listen(sock, 5) < 0) {
-		ERROR("problems listen'ing: %s\n", strerror(errno));
-		exit(EXIT_FAILURE);
-	}
+	if (listen(sock, 5) < 0)
+		FATAL("problems listen'ing: %s\n", strerror(errno));
 
 	numberOfListenSockets++;
 	listenSockets =
@@ -110,8 +105,8 @@ static int establishListen(unsigned int port,
 
 static void parseListenConfigParam(unsigned int port, ConfigParam * param)
 {
-	struct sockaddr *addrp;
-	socklen_t addrlen;
+	struct sockaddr *addrp = NULL;
+	socklen_t addrlen = 0;
 	struct sockaddr_in sin;
 #ifdef HAVE_IPV6
 	struct sockaddr_in6 sin6;
@@ -132,10 +127,8 @@ static void parseListenConfigParam(unsigned int port, ConfigParam * param)
 			sin6.sin6_addr = in6addr_any;
 			addrp = (struct sockaddr *)&sin6;
 			addrlen = sizeof(struct sockaddr_in6);
-			if (establishListen(port, addrp, addrlen) < 0) {
+			if (establishListen(port, addrp, addrlen) < 0)
 				BINDERROR();
-				exit(EXIT_FAILURE);
-			}
 		}
 #endif
 		sin.sin_addr.s_addr = INADDR_ANY;
@@ -147,24 +140,21 @@ static void parseListenConfigParam(unsigned int port, ConfigParam * param)
 		if (establishListen(port, addrp, addrlen) < 0) {
 #endif
 			BINDERROR();
-			exit(EXIT_FAILURE);
 		}
 	} else {
 		struct hostent *he;
 		DEBUG("binding to address for %s\n", param->value);
 		if (!(he = gethostbyname(param->value))) {
-			ERROR("can't lookup host \"%s\" at line %i\n",
+			FATAL("can't lookup host \"%s\" at line %i\n",
 			      param->value, param->line);
-			exit(EXIT_FAILURE);
 		}
 		switch (he->h_addrtype) {
 #ifdef HAVE_IPV6
 		case AF_INET6:
 			if (!useIpv6) {
-				ERROR("no IPv6 support, but a IPv6 address "
+				FATAL("no IPv6 support, but a IPv6 address "
 				      "found for \"%s\" at line %i\n",
 				      param->value, param->line);
-				exit(EXIT_FAILURE);
 			}
 			memcpy((char *)&sin6.sin6_addr.s6_addr,
 			       (char *)he->h_addr, he->h_length);
@@ -179,15 +169,12 @@ static void parseListenConfigParam(unsigned int port, ConfigParam * param)
 			addrlen = sizeof(struct sockaddr_in);
 			break;
 		default:
-			ERROR("address type for \"%s\" is not IPv4 or IPv6 "
+			FATAL("address type for \"%s\" is not IPv4 or IPv6 "
 			      "at line %i\n", param->value, param->line);
-			exit(EXIT_FAILURE);
 		}
 
-		if (establishListen(port, addrp, addrlen) < 0) {
+		if (establishListen(port, addrp, addrlen) < 0)
 			BINDERROR();
-			exit(EXIT_FAILURE);
-		}
 	}
 }
 
@@ -195,22 +182,20 @@ void listenOnPort(void)
 {
 	int port = DEFAULT_PORT;
 	ConfigParam *param = getNextConfigParam(CONF_BIND_TO_ADDRESS, NULL);
+	ConfigParam *portParam = getConfigParam(CONF_PORT);
 
-	{
-		ConfigParam *portParam = getConfigParam(CONF_PORT);
-
-		if (portParam) {
-			char *test;
-			port = strtol(portParam->value, &test, 10);
-			if (port <= 0 || *test != '\0') {
-				ERROR("%s \"%s\" specified at line %i is not a "
-				      "positive integer", CONF_PORT,
-				      portParam->value, portParam->line);
-				exit(EXIT_FAILURE);
-			}
+	if (portParam) {
+		char *test;
+		port = strtol(portParam->value, &test, 10);
+		if (port <= 0 || *test != '\0') {
+			FATAL("%s \"%s\" specified at line %i is not a "
+			      "positive integer", CONF_PORT,
+			      portParam->value, portParam->line);
 		}
 	}
 
+	boundPort = port;
+
 	do {
 		parseListenConfigParam(port, param);
 	} while ((param = getNextConfigParam(CONF_BIND_TO_ADDRESS, param)));
@@ -266,3 +251,8 @@ void getConnections(fd_set * fds)
 		}
 	}
 }
+
+int getBoundPort(void)
+{
+	return boundPort;
+}
