@@ -1,5 +1,5 @@
 /* the Music Player Daemon (MPD)
- * (c)2003-2006 by Warren Dukes (warren.dukes@gmail.com)
+ * Copyright (C) 2003-2007 by Warren Dukes (warren.dukes@gmail.com)
  * This project's homepage is: http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -29,13 +29,13 @@
 #include <time.h>
 
 static unsigned int logLevel = LOG_LEVEL_LOW;
-static int warningFlushed = 0;
+static int warningFlushed;
 static int stdout_mode = 1;
-static char *warningBuffer = NULL;
+static char *warningBuffer;
 static int out_fd = -1;
 static int err_fd = -1;
-static const char *out_filename = NULL;
-static const char *err_filename = NULL;
+static const char *out_filename;
+static const char *err_filename;
 
 /* redirect stdin to /dev/null to work around a libao bug */
 static void redirect_stdin(void)
@@ -59,7 +59,7 @@ static void redirect_logs(void)
 
 static const char *log_date(void)
 {
-	static char buf[16] = { '\0' };
+	static char buf[16];
 	time_t t = time(NULL);
 	strftime(buf, 16, "%b %d %H:%M : ", localtime(&t));
 	return buf;
@@ -68,7 +68,7 @@ static const char *log_date(void)
 #define BUFFER_LENGTH	4096
 static void buffer_warning(const char *fmt, va_list args)
 {
-	char buffer[BUFFER_LENGTH + 1];
+	char buffer[BUFFER_LENGTH];
 	char *tmp = buffer;
 	size_t len = BUFFER_LENGTH;
 
@@ -100,11 +100,10 @@ void flushWarningLog(void)
 	if (warningBuffer != NULL)
 	{
 		while (s != NULL) {
-			char * next = strchr(s, '\n');
-			if (next != NULL) {
-				*next = '\0';
-				next++;
-			}
+			char *next = strchr(s, '\n');
+			if (next == NULL) break;
+			*next = '\0';
+			next++;
 			fprintf(stderr, "%s\n", s);
 			s = next;
 		}
