@@ -1,5 +1,5 @@
 /* the Music Player Daemon (MPD)
- * (c)2003-2006 by Warren Dukes (warren.dukes@gmail.com)
+ * Copyright (C) 2003-2007 by Warren Dukes (warren.dukes@gmail.com)
  * This project's homepage is: http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -25,16 +25,33 @@
 
 #include <stdlib.h>
 
+#ifdef HAVE_LIBSAMPLERATE
+#include <samplerate.h>
+#endif
+
+typedef struct _ConvState {
+#ifdef HAVE_LIBSAMPLERATE
+	SRC_STATE *state;
+	SRC_DATA data;
+	size_t dataInSize;
+	size_t dataOutSize;
+	mpd_sint8 lastChannels;
+	mpd_sint32 lastInSampleRate;
+	mpd_sint32 lastOutSampleRate;
+	int error;
+#endif
+} ConvState;
+
 void pcm_volumeChange(char *buffer, int bufferSize, AudioFormat * format,
-		      int volume);
+                      int volume);
 
 void pcm_mix(char *buffer1, char *buffer2, size_t bufferSize1,
-	     size_t bufferSize2, AudioFormat * format, float portion1);
+             size_t bufferSize2, AudioFormat * format, float portion1);
 
-void pcm_convertAudioFormat(AudioFormat * inFormat, char *inBuffer, size_t
-			    inSize, AudioFormat * outFormat, char *outBuffer);
+size_t pcm_convertAudioFormat(AudioFormat * inFormat, char *inBuffer,
+                              size_t inSize, AudioFormat * outFormat,
+                              char *outBuffer, ConvState *convState);
 
-size_t pcm_sizeOfOutputBufferForAudioFormatConversion(AudioFormat * inFormat,
-						      size_t inSize,
-						      AudioFormat * outFormat);
+size_t pcm_sizeOfConvBuffer(AudioFormat * inFormat, size_t inSize,
+                            AudioFormat * outFormat);
 #endif
