@@ -1,5 +1,5 @@
 /* the Music Player Daemon (MPD)
- * (c)2003-2006 by Warren Dukes (warren.dukes@gmail.com)
+ * Copyright (C) 2003-2007 by Warren Dukes (warren.dukes@gmail.com)
  * This project's homepage is: http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -138,6 +138,8 @@ int playerInit(void)
 				decode();
 			else if (pc->stop)
 				pc->stop = 0;
+			else if (pc->seek)
+				pc->seek = 0;
 			else if (pc->pause)
 				pc->pause = 0;
 			else if (pc->closeAudio) {
@@ -297,12 +299,12 @@ int getPlayerError(void)
 
 char *getPlayerErrorStr(void)
 {
-	static char *error = NULL;
+	static char *error;
 	int errorlen = MAXPATHLEN + 1024;
 	PlayerControl *pc = &(getPlayerData()->playerControl);
 
-	error = xrealloc(error, errorlen + 1);
-	memset(error, 0, errorlen + 1);
+	error = xrealloc(error, errorlen);
+	error[0] = '\0';
 
 	switch (pc->error) {
 	case PLAYER_ERROR_FILENOTFOUND:
@@ -504,8 +506,8 @@ void playerCycleLogFiles(void)
 /* this actually creates a dupe of the current metadata */
 Song *playerCurrentDecodeSong(void)
 {
-	static Song *song = NULL;
-	static MetadataChunk *prev = NULL;
+	static Song *song;
+	static MetadataChunk *prev;
 	Song *ret = NULL;
 	PlayerControl *pc = &(getPlayerData()->playerControl);
 
