@@ -1,5 +1,5 @@
 /* the Music Player Daemon (MPD)
- * (c)2003-2006 by Warren Dukes (warren.dukes@gmail.com)
+ * Copyright (C) 2003-2007 by Warren Dukes (warren.dukes@gmail.com)
  * This project's homepage is: http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -28,6 +28,7 @@
 #include "utils.h"
 #include "sig_handlers.h"
 #include "state_file.h"
+#include "storedPlaylist.h"
 
 #include <string.h>
 #include <stdlib.h>
@@ -37,8 +38,6 @@
 #include <unistd.h>
 #include <time.h>
 
-#define PLAYLIST_COMMENT	'#'
-
 #define PLAYLIST_STATE_STOP		0
 #define PLAYLIST_STATE_PLAY		1
 
@@ -64,30 +63,15 @@
 #define DEFAULT_PLAYLIST_MAX_LENGTH		(1024*16)
 #define DEFAULT_PLAYLIST_SAVE_ABSOLUTE_PATHS	0
 
-typedef struct _Playlist {
-	Song **songs;
-	/* holds version a song was modified on */
-	mpd_uint32 *songMod;
-	int *order;
-	int *positionToId;
-	int *idToPosition;
-	int length;
-	int current;
-	int queued;
-	int repeat;
-	int random;
-	mpd_uint32 version;
-} Playlist;
-
 static Playlist playlist;
 static int playlist_state = PLAYLIST_STATE_STOP;
 static int playlist_max_length = DEFAULT_PLAYLIST_MAX_LENGTH;
 static int playlist_stopOnError;
-static int playlist_errorCount = 0;
+static int playlist_errorCount;
 static int playlist_queueError;
-static int playlist_noGoToNext = 0;
+static int playlist_noGoToNext;
 
-static int playlist_saveAbsolutePaths = DEFAULT_PLAYLIST_SAVE_ABSOLUTE_PATHS;
+int playlist_saveAbsolutePaths = DEFAULT_PLAYLIST_SAVE_ABSOLUTE_PATHS;
 
 static void swapOrder(int a, int b);
 static int playPlaylistOrderNumber(int fd, int orderNum);
@@ -151,14 +135,13 @@ void initPlaylist(void)
 	if (param) {
 		playlist_max_length = strtol(param->value, &test, 10);
 		if (*test != '\0') {
-			ERROR("max playlist length \"%s\" is not an integer, "
+			FATAL("max playlist length \"%s\" is not an integer, "
 			      "line %i\n", param->value, param->line);
-			exit(EXIT_FAILURE);
 		}
 	}
 
 	playlist_saveAbsolutePaths = getBoolConfigParam(CONF_SAVE_ABSOLUTE_PATHS);
-	if (playlist_saveAbsolutePaths == -1) playlist_saveAbsolutePaths = 0;
+	if (playlist_saveAbsolutePaths == -1) playlist_saveAbsolutePaths = DEFAULT_PLAYLIST_SAVE_ABSOLUTE_PATHS;
 	else if (playlist_saveAbsolutePaths < 0) exit(EXIT_FAILURE);
 
 	playlist.songs = xmalloc(sizeof(Song *) * playlist_max_length);
@@ -236,6 +219,12 @@ int clearPlaylist(int fd)
 	return 0;
 }
 
+int clearStoredPlaylist(int fd, char *utf8file)
+{
+	removeAllFromStoredPlaylistByPath(fd, utf8file);
+	return 0;
+}
+
 int showPlaylist(int fd)
 {
 	int i;
@@ -590,6 +579,34 @@ int addToPlaylist(int fd, char *url, int printId)
 	return addSongToPlaylist(fd, song, printId);
 }
 
+int addToStoredPlaylist(int fd, char *url, char *utf8file)
+{
+	Song *song;
+
+	DEBUG("add to stored playlist: %s\n", url);
+
+	song = getSongFromDB(url);
+	if (song) {
+		appendSongToStoredPlaylistByPath(fd, utf8file, song);
+		return 0;
+	}
+
+	if (!isValidRemoteUtf8Url(url))
+		goto fail;
+
+	song = newSong(url, SONG_TYPE_URL, NULL);
+	if (song) {
+		appendSongToStoredPlaylistByPath(fd, utf8file, song);
+		freeJustSong(song);
+		return 0;
+	}
+
+fail:
+	commandError(fd, ACK_ERROR_NO_EXIST, "\"%s\" is not in the music db"
+	             "or is not a valid url", url);
+	return -1;
+}
+
 int addSongToPlaylist(int fd, Song * song, int printId)
 {
 	int id;
@@ -821,7 +838,7 @@ int stopPlaylist(int fd)
 	return 0;
 }
 
-int playPlaylistOrderNumber(int fd, int orderNum)
+static int playPlaylistOrderNumber(int fd, int orderNum)
 {
 
 	if (playerStop(fd) < 0)
@@ -838,8 +855,7 @@ int playPlaylistOrderNumber(int fd, int orderNum)
 	if (playerPlay(fd, (playlist.songs[playlist.order[orderNum]])) < 0) {
 		stopPlaylist(fd);
 		return -1;
-	} else
-		playlist.current++;
+	} 
 
 	playlist.current = orderNum;
 
@@ -1093,12 +1109,26 @@ int moveSongInPlaylist(int fd, int from, int to)
 				playlist.order[i] = to;
 			}
 		}
-	} else if (playlist.current == from)
-		playlist.current = to;
-	else if (playlist.current > from && playlist.current <= to) {
-		playlist.current--;
-	} else if (playlist.current >= to && playlist.current < from) {
-		playlist.current++;
+	}
+	else
+	{
+		if (playlist.current == from)
+			playlist.current = to;
+		else if (playlist.current > from && playlist.current <= to) {
+			playlist.current--;
+		} else if (playlist.current >= to && playlist.current < from) {
+			playlist.current++;
+		}
+
+		/* this first if statement isn't necessary since the queue
+		 * would have been cleared out if queued == from */
+		if (playlist.queued == from)
+			playlist.queued = to;
+		else if (playlist.queued > from && playlist.queued <= to) {
+			playlist.queued--;
+		} else if (playlist.queued>= to && playlist.queued < from) {
+			playlist.queued++;
+		}
 	}
 
 	incrPlaylistVersion();
@@ -1200,7 +1230,7 @@ int setPlaylistRandomStatus(int fd, int status)
 
 int previousSongInPlaylist(int fd)
 {
-	static time_t lastTime = 0;
+	static time_t lastTime;
 	time_t diff = time(NULL) - lastTime;
 
 	lastTime += diff;
@@ -1292,59 +1322,17 @@ int deletePlaylist(int fd, char *utf8file)
 
 int savePlaylist(int fd, char *utf8file)
 {
-	FILE *fileP;
-	int i;
-	struct stat st;
-	char *file;
-	char *rfile;
-	char *actualFile;
+	StoredPlaylist *sp = newStoredPlaylist(utf8file, fd, 0);
+	if (!sp)
+		return -1;
 
-	if (strstr(utf8file, "/")) {
-		commandError(fd, ACK_ERROR_ARG,
-			     "cannot save \"%s\", saving playlists to "
-			     "subdirectories is not supported", utf8file);
+	appendPlaylistToStoredPlaylist(sp, &playlist);
+	if (writeStoredPlaylist(sp) != 0) {
+		freeStoredPlaylist(sp);
 		return -1;
 	}
 
-	file = utf8ToFsCharset(utf8file);
-
-	rfile = xmalloc(strlen(file) + strlen(".") +
-		       strlen(PLAYLIST_FILE_SUFFIX) + 1);
-
-	strcpy(rfile, file);
-	strcat(rfile, ".");
-	strcat(rfile, PLAYLIST_FILE_SUFFIX);
-
-	actualFile = rpp2app(rfile);
-
-	free(rfile);
-
-	if (0 == stat(actualFile, &st)) {
-		commandError(fd, ACK_ERROR_EXIST, "a file or directory already "
-			     "exists with the name \"%s\"", utf8file);
-		return -1;
-	}
-
-	while (!(fileP = fopen(actualFile, "w")) && errno == EINTR) ;
-	if (fileP == NULL) {
-		commandError(fd, ACK_ERROR_SYSTEM, "problems opening file");
-		return -1;
-	}
-
-	for (i = 0; i < playlist.length; i++) {
-		if (playlist_saveAbsolutePaths &&
-		    playlist.songs[i]->type == SONG_TYPE_FILE) {
-			fprintf(fileP, "%s\n",
-				  rmp2amp(utf8ToFsCharset
-					  ((getSongUrl(playlist.songs[i])))));
-		} else {
-			fprintf(fileP, "%s\n",
-			        utf8ToFsCharset(getSongUrl(playlist.songs[i])));
-		}
-	}
-
-	while (fclose(fileP) && errno == EINTR) ;
-
+	freeStoredPlaylist(sp);
 	return 0;
 }
 
@@ -1413,154 +1401,99 @@ int getPlaylistSongId(int song)
 	return playlist.positionToId[song];
 }
 
-static int PlaylistIterFunc(int fd, char *utf8file,
-			    void (*IterFunc) (int fd, char *utf8_file,
-					      char **errored_File))
-{
-	FILE *fileP;
-	char s[MAXPATHLEN + 1];
-	int slength = 0;
-	char *temp = utf8ToFsCharset(utf8file);
-	char *rfile = xmalloc(strlen(temp) + strlen(".") +
-			     strlen(PLAYLIST_FILE_SUFFIX) + 1);
-	char *actualFile;
-	char *parent = parentPath(temp);
-	int parentlen = strlen(parent);
-	char *erroredFile = NULL;
-	int tempInt;
-	int commentCharFound = 0;
-
-	strcpy(rfile, temp);
-	strcat(rfile, ".");
-	strcat(rfile, PLAYLIST_FILE_SUFFIX);
-
-	if ((actualFile = rpp2app(rfile)) && isPlaylist(actualFile))
-		free(rfile);
-	else {
-		free(rfile);
-		commandError(fd, ACK_ERROR_NO_EXIST,
-			     "playlist \"%s\" not found", utf8file);
-		return -1;
-	}
-
-	while (!(fileP = fopen(actualFile, "r")) && errno == EINTR) ;
-	if (fileP == NULL) {
-		commandError(fd, ACK_ERROR_SYSTEM,
-			     "problems opening file \"%s\"", utf8file);
-		return -1;
-	}
-
-	while ((tempInt = fgetc(fileP)) != EOF) {
-		s[slength] = tempInt;
-		if (s[slength] == '\n' || s[slength] == '\0') {
-			commentCharFound = 0;
-			s[slength] = '\0';
-			if (s[0] == PLAYLIST_COMMENT) {
-				commentCharFound = 1;
-			}
-			if (strncmp(s, musicDir, strlen(musicDir)) == 0) {
-				strcpy(s, &(s[strlen(musicDir)]));
-			} else if (parentlen) {
-				temp = xstrdup(s);
-				memset(s, 0, MAXPATHLEN + 1);
-				strcpy(s, parent);
-				strncat(s, "/", MAXPATHLEN - parentlen);
-				strncat(s, temp, MAXPATHLEN - parentlen - 1);
-				if (strlen(s) >= MAXPATHLEN) {
-					commandError(fd,
-						     ACK_ERROR_PLAYLIST_LOAD,
-						     "\"%s\" too long", temp);
-					free(temp);
-					while (fclose(fileP)
-					       && errno == EINTR) ;
-					if (erroredFile)
-						free(erroredFile);
-					return -1;
-				}
-				free(temp);
-			}
-			slength = 0;
-			temp = fsCharsetToUtf8(s);
-			if (!temp)
-				continue;
-			if (!commentCharFound) {
-				/* using temp directly should be safe,
-				 * for our current IterFunction set
-				 * but just in case, we copy to s */
-				strcpy(s, temp);
-				IterFunc(fd, s, &erroredFile);
-			}
-		} else if (slength == MAXPATHLEN) {
-			s[slength] = '\0';
-			commandError(fd, ACK_ERROR_PLAYLIST_LOAD,
-				     "line in \"%s\" is too long", utf8file);
-			ERROR("line \"%s\" in playlist \"%s\" is too long\n",
-			      s, utf8file);
-			while (fclose(fileP) && errno == EINTR) ;
-			if (erroredFile)
-				free(erroredFile);
-			return -1;
-		} else if (s[slength] != '\r')
-			slength++;
-	}
-
-	while (fclose(fileP) && errno == EINTR) ;
-
-	if (erroredFile) {
-		commandError(fd, ACK_ERROR_PLAYLIST_LOAD,
-			     "can't add file \"%s\"", erroredFile);
-		free(erroredFile);
-		return -1;
-	}
-
-	return 0;
-}
-
-static void PlaylistInfoPrintInfo(int fd, char *utf8file, char **erroredfile)
-{
-	Song *song = getSongFromDB(utf8file);
-	if (song) {
-		printSongInfo(fd, song);
-	} else {
-		fdprintf(fd, "file: %s\n", utf8file);
-	}
-}
-static void PlaylistInfoPrint(int fd, char *utf8file, char **erroredfile)
-{
-	fdprintf(fd, "file: %s\n", utf8file);
-}
-
-static void PlaylistLoadIterFunc(int fd, char *temp, char **erroredFile)
-{
-	if (!getSongFromDB(temp) && !isRemoteUrl(temp)) {
-
-	} else if ((addToPlaylist(STDERR_FILENO, temp, 0)) < 0) {
-		/* for windows compatibilit, convert slashes */
-		char *temp2 = xstrdup(temp);
-		char *p = temp2;
-		while (*p) {
-			if (*p == '\\')
-				*p = '/';
-			p++;
-		}
-		if ((addToPlaylist(STDERR_FILENO, temp2, 0)) < 0) {
-			if (!*erroredFile) {
-				*erroredFile = xstrdup(temp);
-			}
-		}
-		free(temp2);
-	}
-}
-
 int PlaylistInfo(int fd, char *utf8file, int detail)
 {
-	if (detail) {
-		return PlaylistIterFunc(fd, utf8file, PlaylistInfoPrintInfo);
+	ListNode *node;
+	StoredPlaylist *sp = loadStoredPlaylist(utf8file, fd);
+	if (sp == NULL)
+		return -1;
+
+	node = sp->list->firstNode;
+	while (node != NULL) {
+		char *temp = node->data;
+		int wrote = 0;
+
+		if (detail) {
+			Song *song = getSongFromDB(temp);
+			if (song) {
+				printSongInfo(fd, song);
+				wrote = 1;
+			}
+		}
+
+		if (!wrote) {
+			fdprintf(fd, SONG_FILE "%s\n", temp);
+		}
+
+		node = node->nextNode;
 	}
-	return PlaylistIterFunc(fd, utf8file, PlaylistInfoPrint);
+
+	freeStoredPlaylist(sp);
+	return 0;
 }
 
 int loadPlaylist(int fd, char *utf8file)
 {
-	return PlaylistIterFunc(fd, utf8file, PlaylistLoadIterFunc);
+	ListNode *node;
+	StoredPlaylist *sp = loadStoredPlaylist(utf8file, fd);
+	if (sp == NULL)
+		return -1;
+
+	node = sp->list->firstNode;
+	while (node != NULL) {
+		char *temp = node->data;
+		if ((addToPlaylist(STDERR_FILENO, temp, 0)) < 0) {
+			/* for windows compatibility, convert slashes */
+			char *temp2 = xstrdup(temp);
+			char *p = temp2;
+			while (*p) {
+				if (*p == '\\')
+					*p = '/';
+				p++;
+			}
+			if ((addToPlaylist(STDERR_FILENO, temp2, 0)) < 0) {
+				commandError(fd, ACK_ERROR_PLAYLIST_LOAD,
+							"can't add file \"%s\"", temp2);
+			}
+			free(temp2);
+		}
+
+		node = node->nextNode;
+	}
+
+	freeStoredPlaylist(sp);
+	return 0;
+}
+
+void searchForSongsInPlaylist(int fd, int numItems, LocateTagItem * items)
+{
+	int i;
+	char **originalNeedles = xmalloc(numItems * sizeof(char *));
+
+	for (i = 0; i < numItems; i++) {
+		originalNeedles[i] = items[i].needle;
+		items[i].needle = strDupToUpper(originalNeedles[i]);
+	}
+
+	for (i = 0; i < playlist.length; i++) {
+		if (strstrSearchTags(playlist.songs[i], numItems, items))
+			printPlaylistSongInfo(fd, i);
+	}
+
+	for (i = 0; i < numItems; i++) {
+		free(items[i].needle);
+		items[i].needle = originalNeedles[i];
+	}
+
+	free(originalNeedles);
+}
+
+void findSongsInPlaylist(int fd, int numItems, LocateTagItem * items)
+{
+	int i;
+
+	for (i = 0; i < playlist.length; i++) {
+		if (tagItemsFoundAndMatches(playlist.songs[i], numItems, items))
+			printPlaylistSongInfo(fd, i);
+	}
 }
