@@ -1,5 +1,5 @@
 /* the Music Player Daemon (MPD)
- * (c)2003-2006 by Warren Dukes (warren.dukes@gmail.com)
+ * Copyright (C) 2003-2007 by Warren Dukes (warren.dukes@gmail.com)
  * This project's homepage is: http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,14 +21,31 @@
 
 #include "../config.h"
 
-#include "song.h"
-#include "mpd_types.h"
+#include "dbUtils.h"
 
 #include <stdio.h>
 #include <sys/param.h>
 #include <time.h>
 
 #define PLAYLIST_FILE_SUFFIX 	"m3u"
+#define PLAYLIST_COMMENT	'#'
+
+typedef struct _Playlist {
+	Song **songs;
+	/* holds version a song was modified on */
+	mpd_uint32 *songMod;
+	int *order;
+	int *positionToId;
+	int *idToPosition;
+	int length;
+	int current;
+	int queued;
+	int repeat;
+	int random;
+	mpd_uint32 version;
+} Playlist;
+
+extern int playlist_saveAbsolutePaths;
 
 void initPlaylist(void);
 
@@ -40,8 +57,12 @@ void savePlaylistState(FILE *);
 
 int clearPlaylist(int fd);
 
+int clearStoredPlaylist(int fd, char *utf8file);
+
 int addToPlaylist(int fd, char *file, int printId);
 
+int addToStoredPlaylist(int fd, char *file, char *utf8file);
+
 int addSongToPlaylist(int fd, Song * song, int printId);
 
 int showPlaylist(int fd);
@@ -116,4 +137,8 @@ int playlistChangesPosId(int fd, mpd_uint32 version);
 
 int PlaylistInfo(int fd, char *utf8file, int detail);
 
+void searchForSongsInPlaylist(int fd, int numItems, LocateTagItem * items);
+
+void findSongsInPlaylist(int fd, int numItems, LocateTagItem * items);
+
 #endif
