@@ -1,5 +1,5 @@
 /* the Music Player Daemon (MPD)
- * (c)2003-2006 by Warren Dukes (warren.dukes@gmail.com
+ * Copyright (C) 2003-2007 by Warren Dukes (warren.dukes@gmail.com)
  * This project's homepage is: http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -28,6 +28,7 @@
 #include "tagTracker.h"
 #include "mpd_types.h"
 #include "gcc.h"
+#include "song.h"
 
 #include <sys/stat.h>
 #include <stdlib.h>
@@ -41,6 +42,9 @@
 #  ifndef ID3_FRAME_COMPOSER
 #    define ID3_FRAME_COMPOSER "TCOM"
 #  endif
+#  ifndef ID3_FRAME_PERFORMER
+#    define ID3_FRAME_PERFORMER "TOPE"
+#  endif
 #  ifndef ID3_FRAME_DISC
 #    define ID3_FRAME_DISC "TPOS"
 #  endif
@@ -99,9 +103,8 @@ void initTagConfig(void)
 				}
 			}
 			if (strlen(c) && i == TAG_NUM_OF_ITEM_TYPES) {
-				ERROR("error parsing metadata item \"%s\" at "
+				FATAL("error parsing metadata item \"%s\" at "
 				      "line %i\n", c, param->line);
-				exit(EXIT_FAILURE);
 			}
 			s++;
 			c = s;
@@ -112,12 +115,22 @@ void initTagConfig(void)
 	free(temp);
 }
 
+void printTagTypes(int fd)
+{
+	int i;
+
+	for (i = 0; i < TAG_NUM_OF_ITEM_TYPES; i++) {
+		if (ignoreTagItems[i] == 0)
+			fdprintf(fd, "tagtype: %s\n", mpdTagItemKeys[i]);
+	}
+}
+
 void printMpdTag(int fd, MpdTag * tag)
 {
 	int i;
 
 	if (tag->time >= 0)
-		fdprintf(fd, "Time: %i\n", tag->time);
+		fdprintf(fd, SONG_TIME "%i\n", tag->time);
 
 	for (i = 0; i < tag->numOfItems; i++) {
 		fdprintf(fd, "%s: %s\n", mpdTagItemKeys[tag->items[i].type],
@@ -195,6 +208,7 @@ MpdTag *parseId3Tag(struct id3_tag * tag)
 	ret = getID3Info(tag, ID3_FRAME_YEAR, TAG_ITEM_DATE, ret);
 	ret = getID3Info(tag, ID3_FRAME_GENRE, TAG_ITEM_GENRE, ret);
 	ret = getID3Info(tag, ID3_FRAME_COMPOSER, TAG_ITEM_COMPOSER, ret);
+	ret = getID3Info(tag, ID3_FRAME_PERFORMER, TAG_ITEM_PERFORMER, ret);
 	ret = getID3Info(tag, ID3_FRAME_COMMENT, TAG_ITEM_COMMENT, ret);
 	ret = getID3Info(tag, ID3_FRAME_DISC, TAG_ITEM_DISC, ret);
 
@@ -421,6 +435,8 @@ MpdTag *apeDup(char *file)
 
 	/* read tag into buffer */
 	tagLen -= sizeof(footer);
+	if (tagLen <= 0)
+		goto fail;
 	buffer = xmalloc(tagLen);
 	if (fread(buffer, 1, tagLen, fp) != tagLen)
 		goto fail;
@@ -483,9 +499,8 @@ MpdTag *newMpdTag(void)
 
 static void deleteItem(MpdTag * tag, int index)
 {
-	tag->numOfItems--;
-
 	assert(index < tag->numOfItems);
+	tag->numOfItems--;
 
 	removeTagItemString(tag->items[index].type, tag->items[index].value);
 	/* free(tag->items[index].value); */
