@@ -1,5 +1,5 @@
 /* the Music Player Daemon (MPD)
- * (c)2003-2006 by Warren Dukes (warren.dukes@gmail.com)
+ * Copyright (C) 2003-2007 by Warren Dukes (warren.dukes@gmail.com)
  * This project's homepage is: http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -73,7 +73,7 @@ static int volume_ossControl = SOUND_MIXER_PCM;
 #endif
 
 #ifdef HAVE_ALSA
-static snd_mixer_t *volume_alsaMixerHandle = NULL;
+static snd_mixer_t *volume_alsaMixerHandle;
 static snd_mixer_elem_t *volume_alsaElem;
 static long volume_alsaMin;
 static long volume_alsaMax;
@@ -430,9 +430,8 @@ void initVolume(void)
 			volume_mixerType = VOLUME_MIXER_TYPE_SOFTWARE;
 			volume_mixerDevice = VOLUME_MIXER_SOFTWARE_DEFAULT;
 		} else {
-			ERROR("unknown mixer type %s at line %i\n",
+			FATAL("unknown mixer type %s at line %i\n",
 			      param->value, param->line);
-			exit(EXIT_FAILURE);
 		}
 	}
 
